#ifndef _SYSLIB_IO_H
#define _SYSLIB_IO_H

/*
	Standard header file for PROGS SMS C library

	File io handling library

	(c) PROGS, Professional & Graphical Software
	written by Joachim Van der Auwera, 19 October 1994
*/

#include <sms.h>
#include <time.h>

typedef int Channel;				/* Channel Identifier type */

#define CHANNEL_NONE (Channel)-1	/* invalid Channel id */

#define IO_MAXNAME		38			/* max length of a file name +1 */
#define IO_MAXDIRECTORY 50			/* max length of a directory name (including device) */
#define IO_MAXFULLNAME	50			/* max length of a file name including directory & device */
#define IO_MAXMEDIUMNAME 22 		/* max length of a medium name +1 */
#define IO_MAXDEVICENAME 8			/* max length of a device name */

#define FOREVER 		(TimeOut)-1
#define NOWAIT			(TimeOut)0
typedef short TimeOut;				/* TimeOut value */

/* constants for io library */
#define OPEN_OLD		(OpenKey)0
#define OPEN_READ		(OpenKey)1
#define OPEN_NEW		(OpenKey)2
#define OPEN_OVER		(OpenKey)3
typedef short OpenKey;				/* key for type of file open */

#define PATH_DATA		(char *)-1
#define PATH_PROG		(char *)-2
#define PATH_DATAPROG	(char *)-3
#define PATH_PROGDATA	(char *)-4

#define TO_END_OF_FILE 0x3fffffff	/* to set/move filepos to end of file */
									/* apparently 1GB is the max file length */

#define CURRENT_TIME (Time)0		/* to set update/backup time to NOW */

#define MAX_VERSION 	(Version)65536
typedef int FileVersion;			/* Channel Version type, limited ! */

#define FILETYPE_NORMAL 			(FileType)0
#define FILETYPE_TEXT				(FileType)0
#define FILETYPE_EXECUTABLE 		(FileType)1
#define FILETYPE_RELOCATABLE		(FileType)2
#define FILETYPE_EXTERNAL_MODULE	(FileType)7
#define FILETYPE_DIRECTORY			(FileType)-1
/* filetypes 3,4,5,6,11 were used by some ancient software */
typedef signed char FileType;

typedef struct {
	char name[IO_MAXNAME];			/* name of file */
	Size length;					/* file length */
	FileType type;					/* type type */
	Size dataspace; 				/* dataspace area size */
	Time UpdateTime;				/* last update time */
	Time BackupTime;				/* last backup time */
	FileVersion version;			/* file version */
	} FileInfo;

#define IO_FORMATTYPE_UNKNOWN	(char)-1
#define IO_FORMATTYPE_QDOS		(char)1
#define IO_FORMATTYPE_MSDOS 	(char)2

#define IO_MEDIUMTYPE_UNKNOWN	(char)-1
#define IO_MEDIUMTYPE_RAM		(char)0
#define IO_MEDIUMTYPE_FLP		(char)1
#define IO_MEDIUMTYPE_HD		(char)2
#define IO_MEDIUMTYPE_CD		(char)3

typedef struct {
	char mediumname[IO_MAXMEDIUMNAME];	/* medium name */
	char devicename[IO_MAXDEVICENAME];	/* device name e.g. "WIN1_" */
	char readonly;					/* TRUE when device is read only */
	Size unitsize;					/* allocation unit size */
	Size mediumsize;				/* total size in bytes */
	Size mediumfree;				/* free space in bytes */
	char formattype, formatsubtype; /* format type and subtype */
	char density;					/* medium density */
	char mediumtype;				/* medium type */
	char removable; 				/* TRUE when possibly removable */
	char unused[15];				/* all -1, for future use */
	} MediumInfo;


Error IOOpen(char *name, OpenKey key, Channel *id);
Error IOOpenFor(char *name, OpenKey key, Job id, Channel *ret);
Error IOOpenPath(char *name, OpenKey key, char *path, Channel *ret);
#define IOOpenPath(n,k,p,c) IOOpenPathFor(n,k,MYSELF,p,c)
Error IOOpenPathFor(char *name, OpenKey key, Job id, char *path, Channel *ret);
Error IOClose(Channel);
Error IOFormat(char *name, int *good, int *total);
Error IODelete(char *name);

Error IOGet(Channel id, char *ret);
Error IOPut(Channel id, char value);
Error IOGetBytes(Channel id, int count, char *place, int *got);
Error IOPutBytes(Channel id, int count, char *bytes, int *sent);
Error IOGetString(Channel id, char *string, Size buffsize);
Error IOPutString(Channel id, char *string, int *sent);
Error IOPutRawBytes(Channel id, int count, char *bytes, int *sent);

Error IOTest(Channel);
Error IODone(Channel);
Error IOFlush(Channel);
Error IOTruncate(Channel);

Error IOTimeOutSet(Channel id, TimeOut t);
Error IOTimeOutGet(Channel id, TimeOut *t);
Error IOPutStringAddNewLineGet(Channel id, char *status);
Error IOPutStringAddNewLineSet(Channel id, char status);

Error IOPositionGet(Channel id, int *position);
Error IOPositionSet(Channel id, int position);
Error IOPositionMove(Channel id, int position);

Error IOFileLoad(Channel id, Size length, void *base);
Error IOFileSave(Channel id, Size length, void *base);

Error IOMedium(Channel id, MediumInfo *info);
Error IORename(Channel id, char *to);

Error IOName(Channel id, char *name, char *directory);
Error IOLength(Channel id, Size *length);
Error IOUpdateTimeGet(Channel id, Time *result);
Error IOUpdateTimeSet(Channel id, Time value);
Error IOBackupTimeGet(Channel id, Time *result);
Error IOBackupTimeSet(Channel id, Time value);
Error IOVersionGet(Channel id, FileVersion *result);
Error IOVersionSet(Channel id, FileVersion value);
Error IOFileTypeGet(Channel id, FileType *type);
Error IOFileTypeSet(Channel id, FileType type);
Error IODataSpaceGet(Channel id, Size *result);
Error IODataSpaceSet(Channel id, Size dataspace);
Error IOFileInfoGet(Channel id, FileInfo *info);
Error IOFileInfoSet(Channel id, FileInfo *info);

Error IOMakeDirectory(char *directory);

#endif _SYSLIB_IO_H
