#ifndef _SYSLIB_JOB_H
#define _SYSLIB_JOB_H

/*
	Standard header file for PROGS SMS C library

	job handling library

	(c) PROGS, Professional & Graphical Software
	written by Joachim Van der Auwera, 14 December 1994
*/

#include <io.h>
#include <win.h>

#define JOB_MAXNAME 64			/* enough room for a jobname string */
#define DEFAULT_STACK 4096		/* default size of a c job stack */
typedef signed char Priority;	/* priority for job 0..127 */

#define PICK_BOTTOMMOST -1		/* key to pick bottommost job */
#define WAKE_BOTTOMMOST -1		/* key to wake bottommost job */

Error JOBMe(Job *result);
Error JOBTree(Job treetop, Job *scan);
Error JOBInfo(Job job, char *name, char *suspended, Priority *priority);

Error JOBActivate(Job job, Priority priority);
Error JOBActivateWait(Job job, Priority priority);
Error JOBWait(Job job, TimeOut timeout);
Error JOBCreate(Job owner, Size code, Size data, void *start, Job *id, void **base);
Error JOBZap(Job job, Error error);
Error JOBRemove(Job job, Error error);
Error JOBPriority(Job job, Priority priority);
Error JOBUnSuspend(Job job);
Error JOBSuspend(Job job, TimeOut timeout, char *flag);

Error JOBCall(Error (*rout)(), char *name, Priority priority, Size stackspace,
		Job *jobid, int paramcount, ...);
Error JOBLoad(char *name, char *path, Priority priority, Job owner, char wait,
		int stringcount, char **strings, int channelcount, Channel *channels,
		Job *jobid);

Error JOBPick(Job job);
Error JOBWake(Job job);

Error JOBEventSend(Job job, WindowEvent event);
#define JOBEventSend(j,e) WINEventSend(j,e)
Error WinEventWait(TimeOut timeout, WindowEvent *event);
#define JOBEventWait(t,e) WINEventWait(t,e)

#endif _SYSLIB_JOB_H
